import java.io.*;
import java.util.*;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;
    Random random = new Random(0x1_1_1_C_A);
    public static final int MAX_N = 100000;//10^5
    public static final int MAX_VAL = 1_000_000_000;

    private String folder;
    private String subtask;
    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setSubtask(String subtask) {
        this.subtask = subtask;
    }

    public void genSamples() throws IOException {
        setFolder("preliminary");
        setSubtask("subtask0");

        System.out.println("==== Generating samples ===");
        print(new Test("#D..D.#DD", "LRLL", "sample1"));
        print(new Test("#D.DD..#..D", "RRRLL", "sample2"));
        testCount = 0;
    }

    public void genHandleTests(int mx, int points) throws IOException {
    }

    public void genSubtask(int n, int m, int points, int groupNo) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + subtaskCount;
        setSubtask(subtask);
        setFolder("tests");

        System.out.println("==== Generating group #" + groupNo + " ===");

        print(empty(n, m));
        print(empty(1, m));

        print(special(n, m));
        print(allDroids(n, m, 'L'));
        print(allDroids(n, m, 'L', 'R'));
        print(allDroidsSmallFix(n, m, 2.0 / n));
        print(allDroidsSmallFix(n, m, 0.001));
        print(allDroidsSmallFix(n, m, 0.01));
        print(allDroidsSmallFix(n, m, 0.03));
        print(allDroidsSmallFix(n, m, 0.05));
        print(allDroidsSmallFix(n, m, 0.05));

        print(halfLine(n, m));
        print(halfLine(n / 2, m / 4, 3 * m / 4));
        print(halfLine(n / 2, m / 4, m / 2));

        print(genRandom(n, m, 0));
        print(genRandom(n, m, 0.5));
        print(genRandom(n, m, 1.0));

        double[] probablyLeft = new double[]{0.4, 0.45, 0.48, 0.4995, 0.4999, 0.5, 0.50099, 0.52, 0.51, 0.6};
        int[] space = new int[]{1, 2, 3, 4, 1, 2, 3, 4, 5, 2};
        for (int i = 0; i < probablyLeft.length; ++i)
            print(genCleverRandom(n, m, probablyLeft[i], 2));

        testsDesc.println(String.format("comment_subtask%d=\"n &lt;= %d, m &lt;= %d\"", subtaskCount, n, m));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
    }

    private Test empty(int n, int m) {
        StringBuilder droids = new StringBuilder();
        StringBuilder commands = new StringBuilder();
        for (int i = 0; i < n; ++i)
            droids.append('.');
        for (int i = 0; i < m; ++i)
            commands.append(random.nextBoolean() ? 'L' : 'R');
        return new Test(droids.toString(), commands.toString(), "empty field");
    }

    private Test special(int n, int m) {
        StringBuilder droids = new StringBuilder();
        for (int i = 0; i < n / 3; ++i) {
            droids.append('#');
            droids.append('D');
            droids.append('.');
        }
        char[] chars = new char[]{'R', 'L'};
        StringBuilder commands = new StringBuilder();
        for (int i = 0; i < m; ++i)
            commands.append(chars[i % 2]);
        return new Test(droids.toString(), commands.toString(), "#D.#D....");
    }

    private Test genCleverRandom(int n, int m, double p, int space) {
        StringBuilder commands = new StringBuilder();
        int minLeft = 0, maxRight = 0, pos = 0;
        for (int i = 0; i < m; ++i)
            if (random.nextDouble() < p) {
                minLeft = Math.min(minLeft, --pos);
                commands.append('L');
            } else {
                maxRight = Math.max(maxRight, ++pos);
                commands.append('R');
            }
        StringBuilder droids = new StringBuilder();
        for (int i = 0; i < m; ++i)
            droids.append('.');

        int len = maxRight - minLeft;
        System.out.println("LEN = " + len);
        int lastDiez = -1;
        int cntDiezs = 0;
        while (lastDiez + 2 * len + 1 < n) {
            cntDiezs++;
            lastDiez = randInterval(lastDiez + len + 1, Math.min(n - 1, lastDiez + (1 + space) * len));
            droids.setCharAt(lastDiez, '#');
        }
        for (int i = 0; i < n; ++i)
            if (droids.charAt(i) == '.') {
                if (random.nextBoolean())
                    droids.setCharAt(i, 'D');
            }
        return new Test(droids.toString(), commands.toString(), String.format("clever random n = %d, m = %d, p = %f, diezs = %d", n, m, p, cntDiezs));
    }

    private int randInterval(int l, int r) {
        return random.nextInt(r - l + 1) + l;
    }

    private String allDroidsString(int n) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < n; ++i)
            sb.append('D');
        return sb.toString();
    }

    private Test halfLine(int n, int m) {
        return halfLine(n, m / 2, m / 2);
    }

    private Test halfLine(int n, int m1, int m2) {
        String droids = allDroidsString(n);
        StringBuilder commands = new StringBuilder();
        for (int i = 0; i < m1; ++i)
            commands.append('L');
        for (int i = 0; i < m2; ++i)
            commands.append('R');
        return new Test(droids, commands.toString(), String.format("n = %d, m = %d, test = LLLL...RRRR", n, m1 + m2));
    }

    private Test allDroidsSmallFix(int n, int m, double part) {
        StringBuilder commands = new StringBuilder();
        String droids = allDroidsString(n);
        char[] chars = new char[]{'L', 'R'};
        for (int i = 0; i < m; ++i)
            commands.append(chars[i % chars.length]);
        int cnt = (int)(m * part);
        for (int i = 0; i < cnt; ++i) {
            int id = random.nextInt(m);
            if (commands.charAt(id) == 'L')
                commands.setCharAt(id, 'R');
            else
                commands.setCharAt(id, 'L');
        }
        return new Test(droids, commands.toString(), "all robots with small fix");
    }

    private Test allDroids(int n, int m, char... chars) {
        StringBuilder commands = new StringBuilder();
        for (int i = 0; i < m; ++i)
            commands.append(chars[i % chars.length]);
        return new Test(allDroidsString(n), commands.toString(), "all robots, chars = " + arrToString(chars));
    }

    private String arrToString(char[] chars) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < chars.length; ++i) {
            sb.append(chars[i]);
            if (i + 1 != chars.length)
                sb.append(',');
        }
        return sb.toString();
    }

    private Test genRandom(int n, int m, double prob) {
        StringBuilder a = new StringBuilder();
        StringBuilder b = new StringBuilder();
        for (int i = 0; i < n; ++i)
            if (random.nextDouble() < prob)
                a.append('#');
            else if (random.nextBoolean())
                a.append('D');
            else
                a.append('.');
        for (int i = 0; i < m; ++i)
            if (random.nextBoolean())
                b.append('L');
            else
                b.append('R');
        return new Test(a.toString(), b.toString(), String.format("random test: n = %d, m = %d", n, m));
    }


    class Test {
        public String descr;
        public int n;
        public int m;
        public String a;
        public String b;

        public Test(String a, String b, String descr) {
            this.a = a;
            this.b = b;
            n = a.length();
            m = b.length();
            this.descr = descr;
        }
    }

    public void print(Test cur) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);

        System.out.println("Generating test #" + testCount + ": " + cur.descr);

        out.println(String.format("%d %d", cur.n, cur.m));
        out.println(cur.a);
        out.println(cur.b);

        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, 0, cur.descr));
    }

    public void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genSamples();
        genSubtask(1000, 1000, 47, 1);
        genSubtask(MAX_N, MAX_N, 53, 2);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
